function [ Time, Freq ] = compParams( RRt, RRa, Fs )
%COMPPARAMS Summary of this function goes here
%   Detailed explanation goes here

if nargin < 3
   Fs = 4; % default interpolation freq in RR series
end

xx = RRt(1):1/Fs:RRt(end);
yy = spline(RRt,RRa,xx); % interp (spline) RR 
%plot(RRt, RRa, '--b.', xx, yy, '--r.')

lambda = 500; % corresponds to fc = 0.035 Hz (default)
[ yy_stat ] = priorSmooth( yy, lambda ); % remove VLF wandering (skin temperature)
% yy_stat is the detrended RR series

% time-domain analysis
% (1) mean Heart Rate
Time.mean_HR = 1/nanmean(yy)*60;
% (2) standard deviation of RR intervals (SDNN)
Time.SDNN = std(yy);
% (3) root mean square of successive differences of RR intervals
Time.RMSSD = rms(diff(yy));
% (4) NN50: the number of pairs of successive NNs that differ by more than
% 50 ms.
[~, ind] = find(abs(diff(yy))>50/1000);
Time.NN50 = length(ind);
% pNN50, the proportion of NN50 divided by total number of NNs.
Time.PNN50 = Time.NN50/length(yy)*100;

% spectral analysis, Welch PSD
[psd,f] = pwelch(yy_stat,256,128,1024,Fs);
% very low frequency (VLF, 0-0.04 Hz)
% low frequency (LF, 0.04-0.15 Hz)
% high frequency (HF, 0.15- 0.4 Hz)
[i_004,~] = find(f>0.04,1);
[i_015,~] = find(f>0.15,1);
[i_04,~] = find(f>0.4,1);

VLF = sum(f(1:i_004).*psd(1:i_004))*10^6; % Power unit: ms^2 
LF = sum(f(i_004:i_015).*psd(i_004:i_015))*10^6;
HF = sum(f(i_015:i_04).*psd(i_015:i_04))*10^6;
tPower = sum(f.*psd)*10^6;

nm_LF = LF/(tPower-VLF);
nm_HF = HF/(tPower-VLF);
ratio_LtoH = LF/HF;

Freq.VLF = VLF;
Freq.LF = LF;
Freq.HF = HF;
Freq.nm_LF = nm_LF;
Freq.nm_HF = nm_HF;
Freq.ratio_LtoH = ratio_LtoH;

figure
subplot(211)
plot(xx, yy, '--black.', xx, yy-yy_stat', '--r.');
xlabel('Time (s)')
ylabel('RR intervals (s)')
legend('IBI time series','To-be-removed baseline')

subplot(212)
stem(f(1:i_004), psd(1:i_004)*10^6,'r');hold on;
    stem(f(i_004:i_015), psd(i_004:i_015)*10^6,'b');hold on;
    stem(f(i_015:i_04), psd(i_015:i_04)*10^6,'m');
xlabel('Frequency (Hz)')
ylabel('PSD (ms^2/Hz)')

end

