function [Bootstat, sorted_r, r] = phaseRandomCorr(s1, s2, max)

% function [Bootstat, sorted_r, r] = phaseRandomCorr(s1, s2, max)
% Computes Bootstrap inference on the correlation between time series s1
% and s2 (for two-tailed distribution).

% Note: make sure the time series that you are comparing are row
% vectors and not column vectors.

% Useage:
% Inputs:
%   s1,s2: row vectors with identical length
%   max: Boostrapping iterations, e.g., max = 2000
% Outputs:
%   Bootstat.Corr_mean: estimate of correlation coefficient
%   Bootstat.Corr_cbp95: 95% CI using unadjusted bootstrap percentile
%   Bootstat.Corr_cbp95: 95% CI using centered bootstrap percentile (recommended)
%   sorted_r: sorted and positively converted boostrapping corr coefs
%   r: original boostrapping corr coefs

r = zeros(1, max); % pre-allocation

% Monte Carlo approach: phase-randomizer statistical test (Ebisuzaki, 1997);
% Resamping in frequency domain. 
% The advantage of preserving the power spectrum is that 
% resampled series retains the same autocorrelation as the original series.
for mc_loop = 1:max
     time_series1 = s1;
     time_series1 = time_series1';
     time_series2 = s2;
     time_series2 = time_series2';

     rand_phase_half = 2*pi*rand(size(time_series1,1),...
                       round(size(time_series1,2)/2))-pi;

     l1 = length(time_series1);
     l2 = length(time_series2);
     if l1~=l2
         error ('Error: lengths of time series do not match!')
     end

     if mod(l1, 2) == 0 % check length is odd or even
         rand_phase = [rand_phase_half 0 -fliplr(rand_phase_half(:,2:end))];
     else rand_phase = [rand_phase_half -fliplr(rand_phase_half(:,2:end))];
     end

     magnitude = abs(fft(time_series2));

     n = magnitude .* exp(1i.*rand_phase);
     time_series2_randomized = real(ifft(n)); % reconstruct a random-phase series
     r(mc_loop) = corr(time_series1', time_series2_randomized');
end

sorted_r = sort(abs(r)); % negative coefficients are functionally 
                         % converted to their + counterparts;
m = sorted_r(round(0.5*length(sorted_r))); % mean of Bootstrap corr coefficients

% The 95% confidence interval of the correlation by the bootstrap
% percentile method:
limits_u = round(0.975*length(sorted_r));
limits_l = round(0.025*length(sorted_r));
foo_u = sorted_r(limits_u);
foo_l = sorted_r(limits_l);

% The 95% confidence interval of the correlation by the centered bootstrap
% percentile method (robust against skewed Boostrapping distribution):
centered_l = 2*m - foo_u;
centered_u = 2*m - foo_l;

Bootstat.Corr_mean = m;
Bootstat.Corr_bp95 = [foo_l foo_u];
Bootstat.Corr_cbp95 = [centered_l centered_u];
hist(r);figure(gcf); % plot histogram of Boostrapping statistics

end
